@@ -38,7 +38,7 @@ predict.densityMclust <- function(object, newdata, what = c("dens", "cdens"), ..
   return(d)
 }
 
-plot.densityMclust <- function(x, data = NULL, what = c("density", "BIC", "diagnostic"), ...) 
+plot.densityMclust <- function(x, data = NULL, what = c("BIC", "density", "diagnostic"), ...) 
 {
   object <- x # Argh.  Really want to use object anyway
 
@@ -79,16 +79,16 @@ plot.densityMclust <- function(x, data = NULL, what = c("density", "BIC", "diagn
       # present menu waiting user choice
       choice <- menu(what, graphics = FALSE, title = title)
       while(choice != 0)
-           { if(what[choice] == "density")    plot.densityMclust.density(...)
-             if(what[choice] == "BIC")        plot.densityMclust.bic(...)
+           { if(what[choice] == "BIC")        plot.densityMclust.bic(...)
+             if(what[choice] == "density")    plot.densityMclust.density(...)
              if(what[choice] == "diagnostic") plot.densityMclust.diagnostic(...)
              # re-present menu waiting user choice
              choice <- menu(what, graphics = FALSE, title = title)
            }
   } 
   else 
-    { if(any(what == "density"))    plot.densityMclust.density(...)
-      if(any(what == "BIC"))        plot.densityMclust.bic(...)
+    { if(any(what == "BIC"))        plot.densityMclust.bic(...)
+      if(any(what == "density"))    plot.densityMclust.density(...)
       if(any(what == "diagnostic")) plot.densityMclust.diagnostic(...)
   }
  
@@ -108,8 +108,7 @@ plotDensityMclust1 <- function(x, data = NULL, hist.col = "lightgrey", hist.bord
   if(is.null(ylab)) 
     ylab <- "Density"
   #
-  xrange <- c(min(object$range) - diff(object$range)/10,
-              max(object$range) + diff(object$range)/10)
+  xrange <- extendrange(r = object$range, f = 0.1)
   xlim <- eval(mc$xlim, parent.frame())
   if(!is.null(xlim)) 
     xrange <- range(xlim)
@@ -122,29 +121,36 @@ plotDensityMclust1 <- function(x, data = NULL, hist.col = "lightgrey", hist.bord
   { h <- hist(data, breaks = breaks, plot = FALSE)
     plot(h, freq = FALSE, col = hist.col, border = hist.border, main = "",
          xlim = range(h$breaks, xrange), 
-         ylim = range(0, ylim, h$density, max(d)+diff(range(d))*0.1),
+         # ylim = range(0, ylim, h$density, max(d)+diff(range(d))*0.1),
+         ylim =  if(!is.null(ylim)) range(ylim) else range(0, h$density, d),
          xlab = xlab, ylab = ylab)
+    box()
     mc[[1]] <- as.name("lines")
-    mc$x <- eval.points; mc$y <- d; mc$type <- "l"
+    mc$x <- eval.points
+    mc$y <- d
+    mc$type <- "l"
     eval(mc, parent.frame())
   }
   else
   { mc[[1]] <- as.name("plot")
-    mc$x <- eval.points; mc$y <- d
-    mc$type <- "l"; mc$xlim <- xlim
-    mc$ylim <- range(0, ylim, max(d)+diff(range(d))*0.1)
-    mc$ylab <- ylab; mc$xlab <- xlab
+    mc$x <- eval.points
+    mc$y <- d
+    mc$type <- "l"
+    mc$xlim <- xlim
+    mc$ylim <- if(!is.null(ylim)) range(ylim) else range(0, d)
+    mc$ylab <- ylab
+    mc$xlab <- xlab
     eval(mc, parent.frame())
   }
   invisible()
 }
 
-plotDensityMclust2 <- function(x, data = NULL, col = grey(0.6), nlevels = 11, levels = NULL, points.col = 1, pch = 1, ...) 
+plotDensityMclust2 <- function(x, data = NULL, nlevels = 11, levels = NULL, col = grey(0.6), points.pch = 1, points.col = 1, points.cex = 0.8, ...) 
 {
   # This function call surfacePlot() with a suitable modification of arguments
   object <- x # Argh.  Really want to use object anyway
   mc <- match.call(expand.dots = TRUE)
-  mc$x <- mc$points.col <- mc$pch <- NULL
+  mc$x <- mc$points.pch <- mc$points.col <- mc$points.cex <- NULL
   mc$nlevels <- nlevels; mc$levels <- levels
   mc$col <- col
   
@@ -162,24 +168,24 @@ plotDensityMclust2 <- function(x, data = NULL, col = grey(0.6), nlevels = 11, le
   #par$variance$d <- 2 # LS: bug?
   par$variance$cholsigma <- par$variance$sigma
   for(k in seq(par$variance$G))
-  { par$variance$cholsigma[,,k] <- chol(par$variance$sigma[,,k]) }
+     { par$variance$cholsigma[,,k] <- chol(par$variance$sigma[,,k]) }
   mc$parameters <- par
   # now surfacePlot() is called
   mc[[1]] <- as.name("surfacePlot")
   out <- eval(mc, parent.frame())
   if(addPoints)
-    points(data, col = points.col, pch = pch)
+    points(data, pch = points.pch, col = points.col, cex = points.cex)
   #
   invisible(out)
 }
 
-plotDensityMclustd <- function(x, data = NULL, col = grey(0.6), nlevels = 11, levels = NULL, points.col = 1, pch = 1, gap = 0.2, ...) 
+plotDensityMclustd <- function(x, data = NULL, nlevels = 11, levels = NULL, col = grey(0.6), points.pch = 1, points.col = 1, points.cex = 0.8, gap = 0.2, ...) 
 {
   # This function call surfacePlot() with a suitable modification of arguments
   
   object <- x # Argh.  Really want to use object anyway
   mc <- match.call(expand.dots = TRUE)
-  mc$x <- mc$points.col <- mc$pch <- mc$gap <- NULL
+  mc$x <- mc$points.pch <- mc$points.col <- mc$points.cex <- mc$gap <- NULL
   mc$nlevels <- nlevels; mc$levels <- levels
   mc$col <- col
   
@@ -227,8 +233,8 @@ plotDensityMclustd <- function(x, data = NULL, col = grey(0.6), nlevels = 11, le
                 out <- eval(mc, parent.frame())
                 box()
                 if(addPoints)
-                  points(data[,c(j,i)], col = points.col, pch = pch)
-    
+                  points(data[,c(j,i)], pch = points.pch, 
+                         col = points.col, cex = points.cex, ...)
               }
               if(i == 1 && (!(j%%2))) axis(3)
               if(i == nc && (j%%2))   axis(1)
